/* (C) 2016 Yana */

module hlsglue (
	input clock,
	input reset,
	input ser_in,
	output ser_out,
	input [11:0] baud_freq,
	input [15:0] baud_limit
);

wire [7:0] rx_data;
wire new_rx_data;
reg [7:0] tx_data;
reg new_tx_data;
wire tx_busy;
wire baud_clk;

reg ap_start;
wire ap_done;
wire ap_idle;
wire ap_ready;
reg [7:0] core_reset;
reg [7:0] core_c;
wire [7:0] core_return;

reg [7:0] in_data;
reg in_ready;
reg [2:0] state;

always @(posedge clock, posedge reset) begin
	if (reset) begin
		new_tx_data <= 0;
		ap_start <= 0;
		in_ready <= 0;
		state <= 0;
	end else begin
		if (new_rx_data) begin
			in_data <= rx_data;
			in_ready <= 1;
		end
		case (state)
			0: begin
				ap_start <= 1;
				core_reset <= 1;
				core_c <= 0;
				state <= 1;
			end
			1: begin
				if (ap_ready) begin
					if (!ap_done) begin
						ap_start <= 0;
						state <= 2;
					end else begin
						if (core_return) begin
							tx_data = core_return;
							ap_start <= 0;
							state <= 3;
						end else begin
							core_reset <= 0;
							if (!in_ready) begin
								core_c <= 0;
							end else begin
								core_c <= in_data;
								if (!new_rx_data) begin
									in_ready <= 0;
								end
							end
						end
					end
				end
			end
			2: begin
				if (ap_done) begin
					if (core_return) begin
						tx_data = core_return;
						state <= 3;
					end else begin
						state <= 5;
					end
				end
			end
			3: begin
				if (!tx_busy) begin
					new_tx_data <= 1;
					state <= 4;
				end
			end
			4: begin
				new_tx_data <= 0;
				state <= 5;
			end
			5: begin
				ap_start <= 1;
				core_reset <= 0;
				if (!in_ready) begin
					core_c <= 0;
				end else begin
					core_c <= in_data;
					if (!new_rx_data) begin
						in_ready <= 0;
					end
				end
				state <= 1;
			end
		endcase
	end
end

uart_top uart_top_0 (
	.clock(clock),
	.reset(reset),
	.ser_in(ser_in),
	.ser_out(ser_out),
	.rx_data(rx_data),
	.new_rx_data(new_rx_data),
	.tx_data(tx_data),
	.new_tx_data(new_tx_data),
	.tx_busy(tx_busy),
	.baud_freq(baud_freq),
	.baud_limit(baud_limit),
	.baud_clk(baud_clk)
);

hlscore hlscore_0 (
	.ap_clk(clock),
	.ap_rst(reset),
	.ap_start(ap_start),
	.ap_done(ap_done),
	.ap_idle(ap_idle),
	.ap_ready(ap_ready),
	.reset(core_reset),
	.c(core_c),
	.ap_return(core_return)
);

endmodule
